import { glob } from 'glob';
import { build } from 'esbuild';
import { mkdirSync, writeFileSync, rmSync } from 'node:fs';
import { sassPlugin } from 'esbuild-sass-plugin';
import Root from './root';
import { getPaths, getStylePath, outDir } from './utils';

mkdirSync(outDir, { recursive: true });

void (async () => {
  const pages = await glob('src/pages/**/*.tsx');

  await Promise.all(
    pages.map(async (page) => {
      const { pagePath, tmpJsPath, outputHtmlPath } = getPaths(page);

      await build({
        entryPoints: [pagePath],
        bundle: true,
        outfile: tmpJsPath,
        jsxImportSource: '@kitajs/html',
        minify: true,
        platform: 'node',
        external: ['esbuild'],
        plugins: [
          sassPlugin({
            filter: /\.module\.scss$/,
            type: 'local-css',
          }),
          sassPlugin({
            filter: /\.scss$/,
            type: 'css',
          }),
        ],
      });

      const { default: Page } = await import(tmpJsPath);
      const html = Page.default();
      let stylePaths: string[] = getStylePath(tmpJsPath, outputHtmlPath);

      writeFileSync(
        outputHtmlPath,
        <Root title="My App" stylePaths={stylePaths} scriptPaths={[]}>
          {html}
        </Root>,
      );
    }),
  );

  rmSync('.tmp', { recursive: true });
})();
